@@ -37,7 +37,9 @@ using System;
 using System.Collections;
 using System.IO;
 using System.Text;
+
 using Beagle.Util;
+using Beagle.Daemon;
 
 internal class RTFControlWordType {
 	
@@ -78,6 +80,8 @@ internal class RTFControlWordType {
 		new RTFControlWordType (Type.MetaDataTag, "company"),
 		new RTFControlWordType (Type.ParaEnd, "par"),
 		new RTFControlWordType (Type.Paragraph, "pard"),
+		new RTFControlWordType (Type.SplSection, "header"),
+		new RTFControlWordType (Type.SplSection, "footer"),
 		new RTFControlWordType (Type.SplSection, "headerl"),
 		new RTFControlWordType (Type.SplSection, "footerl"),
 		new RTFControlWordType (Type.SplSection, "footnote"),
@@ -135,7 +139,8 @@ namespace Beagle.Filters {
 		public FilterRTF ()
 		{
 			// Make this a general rtf filter.
-			AddSupportedMimeType ("application/rtf");
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/rtf"));
+
 			pos = Position.None;
 			groupCount = 0;
 			skipCount = 0;
