@@ -62,73 +62,43 @@ namespace Beagle.Util {
 		public void DebugPrint (string str) {
 			if (!verbose)
 				return;
-			System.Console.WriteLine (str);
+			Logger.Log.Debug ("{0}", str);
 		}
 
-		public class ImBuddyComparer : IComparer {
-			
-			int IComparer.Compare (Object x, Object y) {
-				string accountx, accounty;
-	
-				try {
-					accountx = ((ImBuddy) x).BuddyAccountName;
-				} catch {
-					accountx = "";
-				}
-				
-				try {
-					accounty = ((ImBuddy) y).BuddyAccountName;
-				} catch {
-					accounty = "";
-				}
-	
-				return System.String.Compare (accountx, accounty);
-			}
-		}
-
-		public class ImBuddyAliasComparer : IComparer {
-			
-			int IComparer.Compare (Object x, Object y) {
-				try {
-					return System.String.Compare ((string)x, ((ImBuddy) y).BuddyAccountName);
-				} catch {
-					return System.String.Compare (((ImBuddy) x).BuddyAccountName, (string)y);
-				}
-			}
-		}
 	}
 	
 	public class GaimBuddyListReader : ImBuddyListReader {
 
 		string buddyListPath;
 		string buddyListDir;
-		int myWatchDescriptor = -1;
+		DateTime buddyListLastWriteTime;
+		uint timeoutId;
 
 		public GaimBuddyListReader ()
 		{
 			string home = Environment.GetEnvironmentVariable ("HOME");
 			buddyListDir = Path.Combine (home, ".gaim");
 			buddyListPath = Path.Combine (buddyListDir, "blist.xml");
-
-			Read ();
 			
-			myWatchDescriptor = Inotify.Watch (buddyListPath, Inotify.EventType.CloseWrite);
-			Inotify.Event += OnInotifyEvent;
+			if (File.Exists (buddyListPath))
+				Read ();
+
+			// Poll the file once every minute
+			timeoutId = GLib.Timeout.Add (60000, new GLib.TimeoutHandler (ReadTimeoutHandler));
 		}
 
 		~GaimBuddyListReader ()
 		{
-			Inotify.Event -= OnInotifyEvent;
+			if (timeoutId > 0)
+				GLib.Source.Remove (timeoutId);
 		}
-	
-		protected void OnInotifyEvent (int wd,
-					       string path,
-					       string subitem,
-					       Inotify.EventType type,
-					       uint cookie)
+
+		private bool ReadTimeoutHandler ()
 		{
-			if (wd == myWatchDescriptor)
+			if (File.Exists (buddyListPath))
 				Read ();
+
+			return true;
 		}
 
 		private string Format (string name) {
@@ -137,31 +107,42 @@ namespace Beagle.Util {
 
 		override public void Read ()
 		{
+			// If the file hasn't changed, don't do anything.
+			DateTime last_write = File.GetLastWriteTime (buddyListPath);
+			if (last_write == buddyListLastWriteTime)
+				return;
+
+			buddyListLastWriteTime = last_write;
+
 			buddyList = new Hashtable ();
 
-			XmlDocument accounts = new XmlDocument ();
-			accounts.Load (buddyListPath);
-
-			XmlNodeList contacts = accounts.SelectNodes ("//contact");
-
-			foreach (XmlNode contact in contacts) {
-				string groupalias = "";
+			try {
+				XmlDocument accounts = new XmlDocument ();
+				accounts.Load (buddyListPath);
 				
-				foreach (XmlAttribute attr in contact.Attributes) {
-					if (attr.Name == "alias") {
-						groupalias = attr.Value;
+				XmlNodeList contacts = accounts.SelectNodes ("//contact");
+				
+				foreach (XmlNode contact in contacts) {
+					string groupalias = "";
+					
+					foreach (XmlAttribute attr in contact.Attributes) {
+						if (attr.Name == "alias") {
+							groupalias = attr.Value;
+						}
+					}
+					
+					if (groupalias != "") {
+						foreach (XmlNode buddy in contact.ChildNodes) {
+							AddBuddy (buddy, groupalias);
+						}
 					}
 				}
-
-				if (groupalias != "") {
-					foreach (XmlNode buddy in contact.ChildNodes) {
-						AddBuddy (buddy, groupalias);
-					}
+				
+				foreach (XmlNode buddy in accounts.SelectNodes ("//contact[not(@name)]/buddy")) {
+					AddBuddy (buddy);
 				}
-			}
-			
-			foreach (XmlNode buddy in accounts.SelectNodes ("//contact[not(@name)]/buddy")) {
-				AddBuddy (buddy);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Caught exception while trying to parse Gaim contact list: {0}", ex.Message);
 			}
 		}
 
@@ -310,4 +291,119 @@ namespace Beagle.Util {
 
 	}
 
+	/////////////////////////////////////////////////////////////
+
+	public class KopeteBuddyListReader : ImBuddyListReader {
+
+		string buddyListPath;
+		string buddyListDir;
+		DateTime buddyListLastWriteTime;
+		uint timeoutId;
+
+		public KopeteBuddyListReader ()
+		{
+			buddyListDir = Path.Combine (PathFinder.HomeDir, ".kde/share/apps/kopete");
+			buddyListPath = Path.Combine (buddyListDir, "contactlist.xml");
+			
+			if (File.Exists (buddyListPath))
+				Read ();
+
+			// Poll the file once every minute
+			timeoutId = GLib.Timeout.Add (60000, new GLib.TimeoutHandler (ReadTimeoutHandler));
+		}
+
+		~KopeteBuddyListReader ()
+		{
+			if (timeoutId > 0)
+				GLib.Source.Remove (timeoutId);
+		}
+
+		private bool ReadTimeoutHandler ()
+		{
+			if (File.Exists (buddyListPath))
+				Read ();
+
+			return true;
+		}
+
+		private string Format (string name) {
+			return name.ToLower ().Replace (" ", "");
+		}
+
+		override public void Read ()
+		{
+			// If the file hasn't changed, don't do anything.
+			DateTime last_write = File.GetLastWriteTime (buddyListPath);
+			if (last_write == buddyListLastWriteTime)
+				return;
+
+			buddyListLastWriteTime = last_write;
+
+			buddyList = new Hashtable ();
+
+			try {
+				XmlDocument accounts = new XmlDocument ();
+				accounts.Load (buddyListPath);
+				
+				// Find all xml contact nodes in the contact list
+				foreach (XmlNode contact in accounts.SelectNodes ("//meta-contact"))
+					AddContact (contact);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Caught exception while trying to parse Kopete contact list: {0}", ex.Message);
+			}
+		}
+
+		private void AddContact (XmlNode contact) 
+		{
+			string protocol, owner, other, alias, iconlocation, iconchecksum;
+			
+			protocol = "";
+			owner = "";
+			other = "";
+			alias = "";
+			iconlocation = "";
+			iconchecksum = "";
+
+			// For each and every meta-contact, there can be multiple 
+			// buddy information entries if we have a contact added on
+			// multiple protocols. Loop through them.
+
+ 			foreach (XmlNode plugin_node in contact.SelectNodes ("plugin-data")) {
+				// Determin the protocol
+				XmlAttribute plugin_id_attr = plugin_node.Attributes ["plugin-id"];
+				protocol = plugin_id_attr.Value.Substring (0, plugin_id_attr.Value.Length-8).ToLower ();
+				DebugPrint ("Protocol=" + protocol);
+
+				// Fetch all the buddy properties
+				foreach (XmlNode plugin_data_node in plugin_node.SelectNodes ("plugin-data-field")) { 
+					switch (plugin_data_node.Attributes ["key"].Value) {
+					case "contactId":
+						other = plugin_data_node.InnerText;
+						DebugPrint ("Screen=" + other);
+						break;
+					case "accountId":
+						owner = plugin_data_node.InnerText;
+						DebugPrint ("Account=" + owner);
+						break;
+					case "displayName":
+						alias = plugin_data_node.InnerText;
+						DebugPrint ("Alias=" + alias);
+						break;
+					}
+				}
+				
+				// Replace any earlier buddies with the same screenname
+				// FIXME: Not safe since we can have the same screenname on different accounts.
+				if (buddyList.ContainsKey (Format(other)))
+					buddyList.Remove (Format(other));
+				
+				buddyList.Add (Format(other), new ImBuddy (protocol, owner, Format(other), alias, iconlocation, iconchecksum));
+			}
+		}
+		
+
+		public ImBuddy Search (string buddy) {
+			return (ImBuddy)buddyList[Format(buddy)];
+		}
+	}
 }
