@@ -34,14 +34,16 @@ using System.Text;
 using System.Text.RegularExpressions;
 using System.Threading;
 
-using Mono.Posix;
-
 namespace Beagle.Util {
 
 	public class Inotify {
 
 		public delegate void InotifyCallback (Watch watch, string path, string subitem, string srcpath, EventType type);
 
+		public interface Watch {
+			void Unsubscribe ();
+			void ChangeSubscription (EventType new_mask);
+		}
 		/////////////////////////////////////////////////////////////////////////////////////
 
 		[Flags]
@@ -93,23 +95,81 @@ namespace Beagle.Util {
 			public uint      len;
 		}
 
-		[DllImport ("libinotifyglue")]
-		static extern void inotify_glue_init ();
+		[DllImport ("libbeagleglue")]
+		static extern int inotify_glue_init ();
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern int inotify_glue_watch (int fd, string filename, EventType mask);
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern int inotify_glue_ignore (int fd, int wd);
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern unsafe void inotify_snarf_events (int fd,
-								int timeout_seconds,
 								out int nr,
 								out IntPtr buffer);
 
+		[DllImport ("libbeagleglue")]
+		static extern void inotify_snarf_cancel ();
+
 		/////////////////////////////////////////////////////////////////////////////////////
 
+		static public bool Verbose = false;
+		static private int inotify_fd = -1;
+
+		static Inotify ()
+		{
+			log = Logger.Get ("Inotify");
+
+			if (Environment.GetEnvironmentVariable ("BEAGLE_DISABLE_INOTIFY") != null) {
+				Logger.Log.Debug ("BEAGLE_DISABLE_INOTIFY is set");
+				return;
+			}
+
+			if (Environment.GetEnvironmentVariable ("BEAGLE_INOTIFY_VERBOSE") != null)
+				Inotify.Verbose = true;
+
+			try {
+				inotify_fd = inotify_glue_init ();
+			} catch (EntryPointNotFoundException) {
+				Logger.Log.Info ("Inotify not available on system.");
+				return;
+			}
+
+			if (inotify_fd == -1)
+				Logger.Log.Warn ("Could not initialize inotify");
+		}
+
+		static public bool Enabled {
+			get { return inotify_fd >= 0; }
+		}
+
+		/////////////////////////////////////////////////////////////////////////////////////
+
+#if ! ENABLE_INOTIFY
+
+		// Stubs for systems where inotify is unavailable
+
+		static public Watch Subscribe (string path, InotifyCallback callback, EventType mask)
+		{
+			return null;
+		}
+
+		static public void Start ()
+		{
+			return;
+		}
+
+		static public void Stop ()
+		{
+			return;
+		}
+
+#else // ENABLE_INOTIFY
+
+		/////////////////////////////////////////////////////////////////////////////////////
+		static private ArrayList event_queue = new ArrayList ();
+
 		private class QueuedEvent {
 			public int       Wd;
 			public EventType Type;
@@ -148,39 +208,6 @@ namespace Beagle.Util {
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
-		static private int dev_inotify = -1;
-		static private ArrayList event_queue = new ArrayList ();
-
-		static Inotify ()
-		{
-			log = Logger.Get ("Inotify");
-
-			if (Environment.GetEnvironmentVariable ("BEAGLE_DISABLE_INOTIFY") != null) {
-				Logger.Log.Debug ("BEAGLE_DISABLE_INOTIFY is set");
-				return;
-			}
-
-			if (Environment.GetEnvironmentVariable ("BEAGLE_INOTIFY_VERBOSE") != null)
-				Inotify.Verbose = true;
-
-			inotify_glue_init ();
-
-			dev_inotify = Syscall.open ("/dev/inotify", OpenFlags.O_RDONLY);
-			if (dev_inotify == -1)
-				Logger.Log.Warn ("Could not open /dev/inotify");
-		}
-
-		static public bool Enabled {
-			get { return dev_inotify >= 0; }
-		}
-
-		/////////////////////////////////////////////////////////////////////////////////////
-
-		public interface Watch {
-			void Unsubscribe ();
-			void ChangeSubscription (EventType new_mask);
-		}
-
 		private class WatchInternal : Watch {
 			private InotifyCallback callback;
 			private EventType mask;
@@ -218,13 +245,14 @@ namespace Beagle.Util {
 				if (! this.is_subscribed)
 					return;
 
-				Inotify.ChangeSubscription (watchinfo, this, mask);
+				this.mask = mask;
+				CreateOrModifyWatch (this.watchinfo);
 			}
 
 		}
 
 		private class WatchInfo {
-			public int       Wd;
+			public int       Wd = -1;
 			public string    Path;
 			public bool      IsDirectory;
 			public EventType Mask;
@@ -317,19 +345,9 @@ namespace Beagle.Util {
 				throw new IOException (path);
 
 			lock (watched_by_wd) {
-				bool new_watch_needed = true;
-
 				watched = watched_by_path [path] as WatchInfo;
 
-				if (watched != null) {
-					// We already have a WatchInfo object on this path, so we'll reuse it.
-
-					// Does the existing watch already satisfy the event requirements for this subscriber?
-					if ((watched.Mask & mask) == mask)
-						new_watch_needed = false;
-					else
-						mask |= watched.Mask;
-				} else {
+				if (watched == null) {
 					// We need an entirely new WatchInfo object
 					watched = new WatchInfo ();
 					watched.Path = path;
@@ -343,29 +361,14 @@ namespace Beagle.Util {
 					watched_by_path [watched.Path] = watched;
 				}
 
-				if (new_watch_needed) {
-					int wd = -1;
-
-					// We rely on the behaviour that watching the same inode twice won't result
-					// in the wd value changing.
-					// (no need to worry about watched_by_wd being polluted with stale watches)
-					
-					wd = inotify_glue_watch (dev_inotify, path, mask | base_mask);
-					if (wd < 0) {
-						string msg = String.Format ("Attempt to watch {0} failed!", path);
-						throw new IOException (msg);
-					}
-
-					watched.Wd = wd;
-					watched_by_wd [watched.Wd] = watched;
-				}
-
-				watched.Mask = mask;
 				watched.FilterMask = initial_filter;
 				watched.FilterSeen = 0;
 
 				watch = new WatchInternal (callback, mask_orig, watched);
 				watched.Subscribers.Add (watch);
+
+				CreateOrModifyWatch (watched);
+				watched_by_wd [watched.Wd] = watched;
 			}
 
 			return watch;
@@ -399,10 +402,13 @@ namespace Beagle.Util {
 			watched.Subscribers.Remove (watch);
 
 			// Other subscribers might still be around			
-			if (watched.Subscribers.Count > 0)
+			if (watched.Subscribers.Count > 0) {
+				// Minimize it
+				CreateOrModifyWatch (watched);
 				return;
+			}
 
-			int retval = inotify_glue_ignore (dev_inotify, watched.Wd);
+			int retval = inotify_glue_ignore (inotify_fd, watched.Wd);
 			if (retval < 0) {
 				string msg = String.Format ("Attempt to ignore {0} failed!", watched.Path);
 				throw new IOException (msg);
@@ -412,33 +418,39 @@ namespace Beagle.Util {
 			return;
 		}
 
-		static private void ChangeSubscription (WatchInfo watched, WatchInternal watch, EventType new_mask)
+		// Ensure our watch exists, meets all the subscribers requirements,
+		// and isn't matching any other events that we don't care about.
+		static private void CreateOrModifyWatch (WatchInfo watched)
 		{
-			watch.Mask = new_mask;
+			EventType new_mask = base_mask;
+			foreach (WatchInternal watch in watched.Subscribers)
+				new_mask |= watch.Mask;
+
+			if (watched.Wd >= 0 && watched.Mask == new_mask)
+				return;
+
+			// We rely on the behaviour that watching the same inode twice won't result
+			// in the wd value changing.
+			// (no need to worry about watched_by_wd being polluted with stale watches)
 			
-			// If we ask to watch for new events, reset the watch as necessary.
-			// We take advantage of the fact that watching the same inode again
-			// will not change the watch descriptor.
-			if ((watched.Mask & new_mask) != new_mask) {
-				watched.Mask |= new_mask;
-
-				int new_wd;
-				new_wd = inotify_glue_watch (dev_inotify,
-							     watched.Path,
-							     watched.Mask | base_mask);
-
-				if (watched.Wd != new_wd) {
-					string msg = String.Format ("Watch handle changed unexpectedly!", watched.Path);	
-					throw new IOException (msg);
-				}
+			int wd = -1;
+			wd = inotify_glue_watch (inotify_fd, watched.Path, new_mask);
+			if (wd < 0) {
+				string msg = String.Format ("Attempt to watch {0} failed!", watched.Path);
+				throw new IOException (msg);
 			}
+			if (watched.Wd >= 0 && watched.Wd != wd) {
+				string msg = String.Format ("Watch handle changed unexpectedly!", watched.Path);	
+				throw new IOException (msg);
+			}
+
+			watched.Wd = wd;
+			watched.Mask = new_mask;
 		}
-		
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
 		static Thread snarf_thread = null;
-		static Thread dispatch_thread = null;
 		static bool   running = false;
 
 		static public void Start ()
@@ -454,11 +466,8 @@ namespace Beagle.Util {
 
 				running = true;
 
-				snarf_thread = new Thread (new ThreadStart (SnarfWorker));
-				snarf_thread.Start ();
-
-				dispatch_thread = new Thread (new ThreadStart (DispatchWorker));
-				dispatch_thread.Start ();
+				snarf_thread = ExceptionHandlingThread.Start (new ThreadStart (SnarfWorker));
+				ExceptionHandlingThread.Start (new ThreadStart (DispatchWorker));
 			}
 		}
 
@@ -471,6 +480,8 @@ namespace Beagle.Util {
 				running = false;
 				Monitor.Pulse (event_queue);
 			}
+
+			inotify_snarf_cancel ();
 		}
 
 		static unsafe void SnarfWorker ()
@@ -489,8 +500,7 @@ namespace Beagle.Util {
 				int nr;
 
 				// Will block while waiting for events, but with a 1s timeout.
-				inotify_snarf_events (dev_inotify, 
-						      1, 
+				inotify_snarf_events (inotify_fd, 
 						      out nr,
 						      out buffer);
 
@@ -545,15 +555,16 @@ namespace Beagle.Util {
 		}
 
 
-		static public bool Verbose = false;
-
 		// Update the watched_by_path hash and the path stored inside the watch
 		// in response to a move event.
 		static private void MoveWatch (WatchInfo watch, string name)		
 		{
-			watched_by_path.Remove (watch.Path);
-			watch.Path = name;
-			watched_by_path [watch.Path] = watch;
+			lock (watched_by_wd) {
+
+				watched_by_path.Remove (watch.Path);
+				watch.Path = name;
+				watched_by_path [watch.Path] = watch;
+			}
 
 			if (Verbose)
 				Console.WriteLine ("*** inotify: Moved Watch to {0}", watch.Path);
@@ -561,31 +572,41 @@ namespace Beagle.Util {
 
 		// A directory we are watching has moved.  We need to fix up its path, and the path of
 		// all of its subdirectories, their subdirectories, and so on.
-		static private void HandleMove (string srcpath, string dstpath)
+		static private void HandleMove (string srcpath, string dstparent, string dstname)
 		{
-			WatchInfo start = watched_by_path [srcpath] as WatchInfo;	// not the same as src!
-			if (start == null) {
-				Console.WriteLine ("Lookup failed for {0}", srcpath);
-				return;
-			}
+			string dstpath = Path.Combine (dstparent, dstname);
+			lock (watched_by_wd) {
 
-			// Queue our starting point, then walk its subdirectories, invoking MoveWatch() on
-			// each, repeating for their subdirectories.  The relationship between start, child
-			// and dstpath is fickle and important.
-			Queue queue = new Queue();
-			queue.Enqueue (start);
-			do {
-				WatchInfo target = queue.Dequeue () as WatchInfo;
-				for (int i = 0; i < target.Children.Count; i++) {
-					WatchInfo child = target.Children[i] as WatchInfo;
-					string name = Path.Combine (dstpath, child.Path.Substring (start.Path.Length + 1));
-					MoveWatch (child, name);
-					queue.Enqueue (child);
+				WatchInfo start = watched_by_path [srcpath] as WatchInfo;	// not the same as src!
+				if (start == null) {
+					Logger.Log.Warn ("Lookup failed for {0}", srcpath);
+					return;
 				}
-			} while (queue.Count > 0);
 
-			// Ultimately, fixup the original watch, too
-			MoveWatch (start, dstpath);
+				// Queue our starting point, then walk its subdirectories, invoking MoveWatch() on
+				// each, repeating for their subdirectories.  The relationship between start, child
+				// and dstpath is fickle and important.
+				Queue queue = new Queue();
+				queue.Enqueue (start);
+				do {
+					WatchInfo target = queue.Dequeue () as WatchInfo;
+					for (int i = 0; i < target.Children.Count; i++) {
+						WatchInfo child = target.Children[i] as WatchInfo;
+						Logger.Log.Debug ("Moving watch on {0} from {1} to {2}", child.Path, srcpath, dstpath);
+						string name = Path.Combine (dstpath, child.Path.Substring (srcpath.Length + 1));
+						MoveWatch (child, name);
+						queue.Enqueue (child);
+					}
+				} while (queue.Count > 0);
+				
+				// Ultimately, fixup the original watch, too
+				MoveWatch (start, dstpath);
+				if (start.Parent != null)
+					start.Parent.Children.Remove (start);
+				start.Parent = watched_by_path [dstparent] as WatchInfo;
+				if (start.Parent != null)
+					start.Parent.Children.Add (start);				
+			}
 		}
 
 		static private void SendEvent (WatchInfo watched, string filename, string srcpath, EventType mask)
@@ -609,7 +630,7 @@ namespace Beagle.Util {
 			if (watched.Subscribers == null)
 				return;
 
-			foreach (WatchInternal watch in watched.Subscribers)
+			foreach (WatchInternal watch in (IEnumerable) watched.Subscribers.Clone ())
 				try {
 					if (watch.Callback != null && (watch.Mask & mask) != 0)
 						watch.Callback (watch, watched.Path, filename, srcpath, mask);
@@ -764,9 +785,8 @@ namespace Beagle.Util {
 						srcpath = Path.Combine (paired_watched.Path, next_event.PairedMove.Filename);
 						
 						// Handle the internal rename of the directory.
-						string dstpath = Path.Combine (watched.Path, next_event.Filename);
-						if (Directory.Exists (dstpath))
-							HandleMove (srcpath, dstpath);
+						if ((next_event.Type & EventType.IsDirectory) != 0)
+							HandleMove (srcpath, watched.Path, next_event.Filename);
 					}
 				}
 
@@ -825,5 +845,7 @@ namespace Beagle.Util {
 		}
 #endif
 
+#endif // ENABLE_INOTIFY
+
 	}
 }
