@@ -110,7 +110,7 @@ namespace Beagle.Util {
 							Directory.CreateDirectory (storage_dir);
 							// Make sure that the directory is only
 							// readable by the owner.
-							Mono.Unix.Syscall.chmod (storage_dir, (Mono.Unix.FilePermissions) 448); // 448 == 0700
+							Mono.Unix.Native.Syscall.chmod (storage_dir, (Mono.Unix.Native.FilePermissions) 448); // 448 == 0700
 						}
 					}
 				}
@@ -156,7 +156,7 @@ namespace Beagle.Util {
 							Directory.CreateDirectory (remote_storage_dir);
 							// Make sure that the directory is only
 							// readable by the owner.
-							Mono.Unix.Syscall.chmod (remote_storage_dir, (Mono.Unix.FilePermissions) 448); // 448 == 0700
+							Mono.Unix.Native.Syscall.chmod (remote_storage_dir, (Mono.Unix.Native.FilePermissions) 448); // 448 == 0700
 						}
 					}
 				} else
@@ -180,7 +180,7 @@ namespace Beagle.Util {
 
 						// Make sure that the directory is only readable by the owner. 
 						// Required when using index synchronization as then it resides in /tmp
-						Mono.Unix.Syscall.chmod (index_dir, (Mono.Unix.FilePermissions) 448); // 448 == 0700
+						Mono.Unix.Native.Syscall.chmod (index_dir, (Mono.Unix.Native.FilePermissions) 448); // 448 == 0700
 					}
 				}
 				
