@@ -90,8 +90,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
                                 Logger.Log.Debug ("Starting Inotify Backend");
                                 event_backend = new InotifyBackend ();
                         } else {
-                                Logger.Log.Debug ("Starting FileSystemWatcher Backend");
-                                event_backend = new FileSystemWatcherBackend ();
+                                Logger.Log.Debug ("Creating null file event backend");
+				event_backend = new NullFileEventBackend ();
                         }
 
 			tree_crawl_task = new TreeCrawlTask (new TreeCrawlTask.Handler (AddDirectory));
@@ -144,7 +144,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			string no_ext, ext, no_punct;
 			no_ext = Path.GetFileNameWithoutExtension (name);
-			ext = Path.GetExtension (name);
+			ext = Path.GetExtension (name).ToLower ();
 			
 			sb.Append (no_ext);
 			for (int i = 0; i < sb.Length; ++i)
@@ -913,11 +913,15 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				return RequiredAction.Index;
 			}
 
-			// If the inode ctime is different that the time we last
-			// set file attributes, we might have been moved or copied.
-			if (attr.LastAttrTime != last_attr_time) {
+			// If the inode ctime is newer than the last time we last
+			// set file attributes, we might have been moved.  We don't
+			// strictly compare times due to the fact that although
+			// setting xattrs changes the ctime, if we don't have write
+			// access our metadata will be stored in sqlite, and the
+			// ctime will be at some point in the past.
+			if (attr.LastAttrTime < last_attr_time) {
 				if (Debug)
-					Logger.Log.Debug ("*** CTime has changed, checking last known path ({0} vs {1})", attr.LastAttrTime, last_attr_time);
+					Logger.Log.Debug ("*** CTime is newer, checking last known path ({0} vs {1})", attr.LastAttrTime, last_attr_time);
 
 				last_known_path = UniqueIdToFullPath (attr.UniqueId);
 
