@@ -49,8 +49,6 @@ namespace Beagle.Daemon {
 
 		public delegate bool HitFilter (Hit hit);
 
-		public const string UnindexedNamespace = "_unindexed:";
-
 		// VERSION HISTORY
 		// ---------------
 		//
@@ -74,7 +72,8 @@ namespace Beagle.Daemon {
 		// 11: moved mime type and hit type into properties
 		// 12: added year-month and year-month-day resolutions for all
 		//     date properties
-		private const int MAJOR_VERSION = 12;
+		// 13: moved source into a property
+		private const int MAJOR_VERSION = 13;
 		private int minor_version = 0;
 
 		private string index_name;
@@ -115,7 +114,7 @@ namespace Beagle.Daemon {
 
 		//////////////////////////////////////////////////////////////////////////////
 
-		protected TextCache text_cache = TextCache.UserCache;
+		protected TextCache text_cache = null;
 
 		public TextCache TextCache {
 			get { return text_cache; }
@@ -494,7 +493,7 @@ namespace Beagle.Daemon {
 
 			// Don't actually put properties in the UnindexedNamespace
 			// in the document.  A horrible (and yet lovely!) hack.
-			if (prop.Key.StartsWith (UnindexedNamespace))
+			if (prop.Key.StartsWith (StringFu.UnindexedNamespace))
 				return;
 
 			Field f;
@@ -524,7 +523,7 @@ namespace Beagle.Daemon {
 
 			f = new Field (field_name,
 				       coded_value,
-				       true,        // always store
+				       prop.IsStored,
 				       true,        // always index
 				       true);       // always tokenize (just strips off type code for keywords)
 			doc.Add (f);
@@ -559,6 +558,7 @@ namespace Beagle.Daemon {
 			prop.Value = field_value.Substring (2);
 			prop.IsSearched = (field_value [0] == 's');
 			prop.IsMutable = ! from_primary_index;
+			prop.IsStored = f.IsStored ();
 
 			return prop;
 		}
@@ -654,10 +654,15 @@ namespace Beagle.Daemon {
 				AddPropertyToDocument (prop, primary_doc);
 			}
 
+			if (indexable.Source != null) {
+				Property prop;
+				prop = Property.NewKeyword ("beagle:Source", indexable.Source);
+				AddPropertyToDocument (prop, primary_doc);
+			}
+
 			// Store the other properties
 				
 			foreach (Property prop in indexable.Properties) {
-				
 				Document target_doc = primary_doc;
 				if (prop.IsMutable) {
 					if (secondary_doc == null) {
@@ -738,13 +743,12 @@ namespace Beagle.Daemon {
 			
 			hit.Timestamp = StringFu.StringToDateTime (doc.Get ("Timestamp"));
 
-			hit.Source = "lucene";
-
 			AddPropertiesToHit (hit, doc, true);
 
 			// Get the Type and MimeType from the properties.
 			hit.Type = hit.GetFirstProperty ("beagle:HitType");
 			hit.MimeType = hit.GetFirstProperty ("beagle:MimeType");
+			hit.Source = hit.GetFirstProperty ("beagle:Source");
 
 			return hit;
 		}
@@ -770,9 +774,8 @@ namespace Beagle.Daemon {
 		{
 			if (last_item_count < 0) {
 				IndexReader reader;
-				reader = IndexReader.Open (PrimaryStore);
+				reader = GetReader (PrimaryStore);
 				last_item_count = reader.NumDocs ();
-				reader.Close ();
 			}
 			return last_item_count;
 		}
@@ -1385,6 +1388,68 @@ namespace Beagle.Daemon {
 
 		///////////////////////////////////////////////////////////////////////////////////
 
+		// Cache IndexReaders on a per-Lucene index basis, since they
+		// are extremely expensive to create.  Note that using this
+		// only makes sense in situations where the index only
+		// possibly might change from underneath us, but most of the
+		// time probably won't.  This means it makes sense to do
+		// this in LuceneQueryingDriver.cs, but it doesn't in
+		// LuceneIndexingDriver.cs.
+
+		private class ReaderAndVersion {
+
+			public IndexReader Reader;
+			public long Version;
+
+			public ReaderAndVersion (IndexReader reader, long version)
+			{
+				this.Reader = reader;
+				this.Version = version;
+			}
+		}
+
+		static private Hashtable directory_rav_map = new Hashtable ();
+
+		static public LNS.IndexSearcher GetSearcher (Lucene.Net.Store.Directory directory)
+		{
+			IndexReader reader = GetReader (directory);
+
+			return new LNS.IndexSearcher (reader);
+		}
+
+		static public IndexReader GetReader (Lucene.Net.Store.Directory directory)
+		{
+			IndexReader reader;
+			long version;
+
+			lock (directory_rav_map) {
+				ReaderAndVersion rav = (ReaderAndVersion) directory_rav_map [directory];
+
+				if (rav == null) {
+					version = IndexReader.GetCurrentVersion (directory);
+					reader = IndexReader.Open (directory);
+
+					rav = new ReaderAndVersion (reader, version);
+
+					directory_rav_map [directory] = rav;
+
+					return reader;
+				}
+
+				version = IndexReader.GetCurrentVersion (directory);
+				
+				if (version != rav.Version) {
+					rav.Version = version;
+					rav.Reader.Close ();
+					rav.Reader = IndexReader.Open (directory);
+				}
+				
+				return rav.Reader;
+			}
+		}
+
+		///////////////////////////////////////////////////////////////////////////////////
+
 		//
 		// Various ways to grab lots of hits at once.
 		// These should never be used for querying, only for utility
@@ -1396,8 +1461,8 @@ namespace Beagle.Daemon {
 		{
 			IndexReader primary_reader;
 			IndexReader secondary_reader;
-			primary_reader = IndexReader.Open (PrimaryStore);
-			secondary_reader = IndexReader.Open (SecondaryStore);
+			primary_reader = GetReader (PrimaryStore);
+			secondary_reader = GetReader (SecondaryStore);
 
 			int request_size;
 			request_size = block_of_hits.Length;
@@ -1493,8 +1558,8 @@ namespace Beagle.Daemon {
 
 			IndexReader primary_reader;
 			IndexReader secondary_reader;
-			primary_reader = IndexReader.Open (PrimaryStore);
-			secondary_reader = IndexReader.Open (SecondaryStore);
+			primary_reader = GetReader (PrimaryStore);
+			secondary_reader = GetReader (SecondaryStore);
 
 			// Load everything from the primary index
 			int max_doc;
@@ -1533,10 +1598,6 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			primary_reader.Close ();
-			if (secondary_reader != null)
-				secondary_reader.Close ();
-
 			return all_hits;
 		}
 	}
