@@ -25,6 +25,10 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include "beagle-util.h"
 
 GQuark
@@ -37,3 +41,14 @@ beagle_error_quark (void)
 
 	return quark;
 }
+
+gboolean
+beagle_util_is_path_on_block_device (const char *path)
+{
+	struct stat st;
+
+	if (stat (path, &st) < 0)
+		return FALSE;
+
+	return (st.st_dev >> 8 != 0);
+}
