@@ -26,13 +26,14 @@
 
 using System;
 using System.Collections;
+using System.Globalization;
 using System.Threading;
 using System.Text;
 
 using Gtk;
 
 using Beagle;
-using BU = Beagle.Util;
+using Beagle.Daemon;
 
 class QueryTool {
 
@@ -42,10 +43,13 @@ class QueryTool {
 	static DateTime lastQueryTime = DateTime.Now;
 
 	// CLI args
-	static bool keepRunning = false;
+	static bool keep_running = false;
 	static bool verbose = false;
 	static bool display_hits = true;
 	static bool flood = false;
+	static bool listener = false;
+	static DateTime start_date = DateTime.MinValue;
+	static DateTime end_date = DateTime.MinValue;
 
 	static void OnHitsAdded (HitsAddedResponse response)
 	{
@@ -68,7 +72,7 @@ class QueryTool {
 				Console.WriteLine (hit.Uri);
 
 			if (verbose) {
-				SnippetRequest sreq = new SnippetRequest (query.Text, hit);
+				SnippetRequest sreq = new SnippetRequest (query, hit);
 				SnippetResponse sresp = (SnippetResponse) sreq.Send ();
 				Console.WriteLine ("PaUri: {0}", hit.ParentUri != null ? hit.ParentUri.ToString () : "(null)");
 				Console.WriteLine (" Snip: {0}", sresp.Snippet != null ? sresp.Snippet : "(null)");
@@ -77,7 +81,7 @@ class QueryTool {
 				Console.WriteLine ("  Src: {0}", hit.Source);
 				Console.WriteLine ("Score: {0}", hit.Score);
 				if (hit.ValidTimestamp)
-					Console.WriteLine (" Time: {0}", hit.Timestamp);
+					Console.WriteLine (" Time: {0}", hit.Timestamp.ToLocalTime ());
 				
 				foreach (Property prop in hit.Properties)
 					Console.WriteLine ("    {0} = {1}", prop.Key, prop.Value);
@@ -134,6 +138,10 @@ class QueryTool {
 			"                    \t\ttype. Can be used multiply.\n" +
 			"  --source <source>\t\tConstrain query to the specified source.\n" +
 			"                   \t\tSources list available from beagle-status.\n" +
+			"  --start <date>\t\t\tConstrain query to items after specified date.\n" +
+			"                \t\t\tDate must be in the form \"yyyyMMdd\" or \"yyyyMMddHHmmss\"\n" +
+			"  --end <date>\t\t\tConstrain query to items before specified date.\n" +
+			"              \t\t\tDate must be in the form \"yyyyMMdd\" or \"yyyyMMddHHmmss\"\n" +
 			"  --live-query\t\t\tRun continuously, printing notifications if a\n" +
 			"              \t\t\tquery changes.\n" +
 			"  --stats-only\t\t\tOnly display statistics about the query, not\n" +
@@ -141,10 +149,20 @@ class QueryTool {
 			"  --max-hits\t\t\tLimit number of search results per backend\n" +
 			"            \t\t\t(default = 100, max = 100)\n" +
 			"  --flood\t\t\tExecute the query over and over again.  Don't do that.\n" +
+			"  --listener\t\t\tExecute an index listener query.  Don't do that either.\n" +
 			"  --help\t\t\tPrint this usage message.\n";
 
 		Console.WriteLine (usage);
 
+		Console.WriteLine (
+			"'query string' can include specific keywords e.g. author:jon.\n" +
+			"Supported keywords are:");
+		IDictionaryEnumerator property_keyword_enum = PropertyKeywordFu.MappingEnumerator;
+		while (property_keyword_enum.MoveNext ()) {
+			PropertyDetail prop = property_keyword_enum.Value as PropertyDetail;
+			Console.WriteLine ("  {0,-12} for {1}", property_keyword_enum.Key, prop.Description);
+		}
+
 		System.Environment.Exit (0);
 	}
 
@@ -183,10 +201,15 @@ class QueryTool {
 		if (args.Length == 0 || Array.IndexOf (args, "--help") > -1 || Array.IndexOf (args, "--usage") > -1)
 			PrintUsageAndExit ();
 
-		query = new Query ();
-
 		StringBuilder query_str =  new StringBuilder ();
 
+		string[] formats = {
+			"yyyyMMdd",
+			"yyyyMMddHHmmss"
+		};
+
+		query = new Query ();
+
 		// Parse args
 		int i = 0;
 		while (i < args.Length) {
@@ -205,7 +228,7 @@ class QueryTool {
 				query.AddSource (args [i]);
 				break;
 			case "--live-query":
-				keepRunning = true;
+				keep_running = true;
 				break;
 			case "--verbose":
 				verbose = true;
@@ -221,6 +244,33 @@ class QueryTool {
 			case "--flood":
 				flood = true;
 				break;
+			case "--listener":
+				listener = true;
+				keep_running = true;
+				break;
+			case "--start":
+				if (++i >= args.Length) PrintUsageAndExit ();
+				try {
+					start_date = DateTime.ParseExact (args[i], formats,
+									  CultureInfo.InvariantCulture,
+									  DateTimeStyles.None);
+				} catch (FormatException) {
+					Console.WriteLine ("Invalid start date");
+					System.Environment.Exit (-1);
+				}
+				break;
+
+			case "--end":
+				if (++i >= args.Length) PrintUsageAndExit ();
+				try {
+					end_date = DateTime.ParseExact (args[i], formats,
+									CultureInfo.InvariantCulture,
+									DateTimeStyles.None);
+				} catch (FormatException) {
+					Console.WriteLine ("Invalid end date");
+					System.Environment.Exit (-1);
+				}
+				break;
 
 			default:
 				if (query_str.Length > 0)
@@ -232,14 +282,33 @@ class QueryTool {
 			++i;
 		}
 
-		if (query_str.Length > 0)
-			query.AddText (query_str.ToString ());
+		if (listener) {
+
+			query.IsIndexListener = true;
+
+		} else {
+		
+			if (query_str.Length > 0)
+				query.AddText (query_str.ToString ());
+
+			if (start_date != DateTime.MinValue || end_date != DateTime.MinValue) {
+				QueryPart_DateRange part = new QueryPart_DateRange ();
+				
+				if (start_date != DateTime.MinValue)
+					part.StartDate = start_date;
+
+				if (end_date != DateTime.MinValue)
+					part.EndDate = end_date;
+				
+				query.AddPart (part);
+			}
+		}
 
 		query.HitsAddedEvent += OnHitsAdded;
 		query.HitsSubtractedEvent += OnHitsSubtracted;
 
 
-		if (! keepRunning)
+		if (! keep_running)
 			query.FinishedEvent += OnFinished;
 		else
 			query.ClosedEvent += OnClosed;
